class UnsupportedAxisError(Exception):
    """Raised when unsupported axis argument is used"""
    pass


class UnsupportedSurfaceMovementError(Exception):
    """Raised when unsupported surface movement is used"""
    pass
